@@ -50,11 +50,13 @@
 #include "simulation/solver/epsilon.h"
 #include "simulation/solver/external_input.h"
 #include "linearSystem.h"
-#include "sym_imp_euler.h"
+#include "sym_solver_ssc.h"
+#include "irksco.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
 #include "simulation/solver/embedded_server.h"
 #include "simulation/solver/real_time_sync.h"
 #endif
+#include "simulation/simulation_input_xml.h"
 
 #include "optimization/OptimizerInterface.h"
 
@@ -75,8 +77,8 @@ typedef struct RK4_DATA
 {
   double** work_states;
   int work_states_ndims;
-  double *b;
-  double *c;
+  const double *b;
+  const double *c;
   double h;
 }RK4_DATA;
 
@@ -84,7 +86,7 @@ typedef struct RK4_DATA
 static int euler_ex_step(DATA* data, SOLVER_INFO* solverInfo);
 static int rungekutta_step_ssc(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo);
 static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo);
-static int sym_euler_im_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo);
+static int sym_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo);
 
 static int radau_lobatto_step(DATA* data, SOLVER_INFO* solverInfo);
 
@@ -138,12 +140,9 @@ int solver_main_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
     return retVal;
 #endif
 #ifdef WITH_SUNDIALS
-  case S_RADAU5:
-  case S_RADAU3:
-  case S_RADAU1:
-  case S_LOBATTO2:
-  case S_LOBATTO4:
-  case S_LOBATTO6:
+  case S_IMPEULER:
+  case S_TRAPEZOID:
+  case S_IMPRUNGEKUTTA:
     retVal = radau_lobatto_step(data, solverInfo);
     if(omc_flag[FLAG_SOLVER_STEPS])
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
@@ -162,16 +161,24 @@ int solver_main_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     TRACE_POP
     return retVal;
-  case S_SYM_EULER:
-    retVal = sym_euler_im_step(data, threadData, solverInfo);
+  case S_SYM_SOLVER:
+    retVal = sym_solver_step(data, threadData, solverInfo);
     if(omc_flag[FLAG_SOLVER_STEPS])
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     return retVal;
-  case S_SYM_IMP_EULER:
-    retVal = sym_euler_im_with_step_size_control_step(data, threadData, solverInfo);
+  case S_SYM_SOLVER_SSC:
+    retVal = sym_solver_ssc_step(data, threadData, solverInfo);
     if(omc_flag[FLAG_SOLVER_STEPS])
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     return retVal;
+  case S_IRKSCO:
+  {
+    retVal = irksco_midpoint_rule(data, threadData, solverInfo);
+    if(omc_flag[FLAG_SOLVER_STEPS])
+      data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
+    return retVal;
+  }
+
   }
 
   TRACE_POP
@@ -207,19 +214,32 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
   solverInfo->solverStats = (unsigned int*) calloc(numStatistics, sizeof(unsigned int));
   solverInfo->solverStatsTmp = (unsigned int*) calloc(numStatistics, sizeof(unsigned int));
 
-  /* if FLAG_NOEQUIDISTANT_GRID is set, choose dassl step method */
+  /* if FLAG_NOEQUIDISTANT_GRID is set, choose integrator step method */
   if (omc_flag[FLAG_NOEQUIDISTANT_GRID])
   {
     solverInfo->integratorSteps = 1; /* TRUE */
   }
+  else
+  {
+    solverInfo->integratorSteps = 0;
+  }
+
+  /* set tolerance for ZeroCrossings */
+  /*  TODO: Check this! */
+  /*  setZCtol(fmin(simInfo->stepSize, simInfo->tolerance)); */
 
   switch (solverInfo->solverMethod)
   {
-  case S_SYM_EULER:
+  case S_SYM_SOLVER:
   case S_EULER: break;
-  case S_SYM_IMP_EULER:
+  case S_SYM_SOLVER_SSC:
   {
-    allocateSymEulerImp(solverInfo, data->modelData->nStates);
+    allocateSymSolverSsc(solverInfo, data->modelData->nStates);
+    break;
+  }
+  case S_IRKSCO:
+  {
+    allocateIrksco(solverInfo, data->modelData->nStates, data->modelData->nZeroCrossings);
     break;
   }
   case S_ERKSSC:
@@ -280,58 +300,37 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
   }
 #endif
 #ifdef WITH_SUNDIALS
-  case S_RADAU5:
+  case S_IMPEULER:
+  case S_TRAPEZOID:
+  case S_IMPRUNGEKUTTA:
   {
-    /* Allocate Radau5 IIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Radau IIA of order 5");
+    int usedImpRKOrder = DEFAULT_IMPRK_ORDER;
+    if (solverInfo->solverMethod == S_IMPEULER)
+      usedImpRKOrder = 1;
+    if (solverInfo->solverMethod == S_TRAPEZOID)
+      usedImpRKOrder = 2;
+
+    /* Check the order if set */
+    if (omc_flag[FLAG_IMPRK_ORDER])
+    {
+      usedImpRKOrder = atoi(omc_flagValue[FLAG_IMPRK_ORDER]);
+      if (usedImpRKOrder>6 || usedImpRKOrder<1)
+      {
+        warningStreamPrint(LOG_STDOUT, 0, "Selected order %d is out of range[1-6]. Use default order %d", usedImpRKOrder, DEFAULT_IMPRK_ORDER);
+        usedImpRKOrder = DEFAULT_IMPRK_ORDER;
+      }
+    }
+
+    /* Allocate implicit Runge-Kutta methods */
+    infoStreamPrint(LOG_SOLVER, 0, "Initializing Runge-Kutta method with order %d", usedImpRKOrder);
     solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 3);
-    break;
-  }
-  case S_RADAU3:
-  {
-    /* Allocate Radau3 IIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Radau IIA of order 3");
-    solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 2);
-    break;
-  }
-  case S_RADAU1:
-  {
-    /* Allocate Radau1 IIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Radau IIA of order 1 (implicit euler) ");
-    solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 1);
-    break;
-  }
-  case S_LOBATTO6:
-  {
-    /* Allocate Lobatto2 IIIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Lobatto IIIA of order 6");
-    solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 3);
-    break;
-  }
-  case S_LOBATTO4:
-  {
-    /* Allocate Lobatto4 IIIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Lobatto IIIA of order 4");
-    solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 2);
-    break;
-  }
-  case S_LOBATTO2:
-  {
-    /* Allocate Lobatto6 IIIA work arrays */
-    infoStreamPrint(LOG_SOLVER, 0, "Initializing Lobatto IIIA of order 2 (trapeze rule)");
-    solverInfo->solverData = calloc(1, sizeof(KINODE));
-    allocateKinOde(data, threadData, solverInfo, solverInfo->solverMethod, 1);
+    allocateKinOde(data, threadData, solverInfo, usedImpRKOrder);
     break;
   }
   case S_IDA:
   {
     IDA_SOLVER* idaData = NULL;
-  /* Allocate Lobatto6 IIIA work arrays */
+    /* Allocate ida working data */
     infoStreamPrint(LOG_SOLVER, 0, "Initializing IDA DAE Solver");
     idaData = (IDA_SOLVER*) malloc(sizeof(IDA_SOLVER));
     retValue = ida_solver_initial(data, threadData, solverInfo, idaData);
@@ -365,9 +364,9 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
   free(solverInfo->solverStats);
   free(solverInfo->solverStatsTmp);
   /* deintialize solver related workspace */
-  if (solverInfo->solverMethod == S_SYM_IMP_EULER)
+  if (solverInfo->solverMethod == S_SYM_SOLVER_SSC)
   {
-    freeSymEulerImp(solverInfo);
+    freeSymSolverSsc(solverInfo);
   }
   else if(solverInfo->solverMethod == S_RUNGEKUTTA || solverInfo->solverMethod == S_HEUN
          || solverInfo->solverMethod == S_ERKSSC)
@@ -378,6 +377,10 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
     free(((RK4_DATA*)(solverInfo->solverData))->work_states);
     free((RK4_DATA*)solverInfo->solverData);
   }
+  else if (solverInfo->solverMethod == S_IRKSCO)
+  {
+    freeIrksco(solverInfo);
+  }
 #if !defined(OMC_MINIMAL_RUNTIME)
   else if(solverInfo->solverMethod == S_DASSL)
   {
@@ -393,35 +396,12 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
   }
 #endif
 #ifdef WITH_SUNDIALS
-  else if(solverInfo->solverMethod == S_RADAU5)
+  else if(solverInfo->solverMethod == S_IMPEULER ||
+          solverInfo->solverMethod == S_TRAPEZOID ||
+          solverInfo->solverMethod == S_IMPRUNGEKUTTA)
   {
     /* free  work arrays */
-    freeKinOde(data, solverInfo, 3);
-  }
-  else if(solverInfo->solverMethod == S_RADAU3)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo, 2);
-  }
-  else if(solverInfo->solverMethod == S_RADAU1)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo, 1);
-  }
-  else if(solverInfo->solverMethod == S_LOBATTO6)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo, 3);
-  }
-  else if(solverInfo->solverMethod == S_LOBATTO4)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo, 2);
-  }
-  else if(solverInfo->solverMethod == S_LOBATTO2)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo, 1);
+    freeKinOde(data, solverInfo);
   }
   else if(solverInfo->solverMethod == S_IDA)
   {
@@ -438,18 +418,17 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
 
 
 /*! \fn initializeModel(DATA* data, const char* init_initMethod,
- *   const char* init_file, double init_time, int lambda_steps)
+ *   const char* init_file, double init_time)
  *
  *  \param [ref] [data]
  *  \param [in]  [pInitMethod] user defined initialization method
  *  \param [in]  [pInitFile] extra argument for initialization-method "file"
  *  \param [in]  [initTime] extra argument for initialization-method "file"
- *  \param [in]  [lambda_steps] ???
  *
  *  This function starts the initialization process of the model .
  */
 int initializeModel(DATA* data, threadData_t *threadData, const char* init_initMethod,
-    const char* init_file, double init_time, int lambda_steps)
+    const char* init_file, double init_time)
 {
   TRACE_PUSH
   int retValue = 0;
@@ -473,9 +452,6 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
 
   data->localData[0]->timeValue = simInfo->startTime;
 
-  /* instance all external Objects */
-  data->callback->callExternalObjectConstructors(data, threadData);
-
   threadData->currentErrorStage = ERROR_SIMULATION;
   /* try */
   {
@@ -483,7 +459,7 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
 #if !defined(OMC_EMCC)
     MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
-    if(initialization(data, threadData, init_initMethod, init_file, init_time, lambda_steps))
+    if(initialization(data, threadData, init_initMethod, init_file, init_time))
     {
       warningStreamPrint(LOG_STDOUT, 0, "Error in initialization. Storing results and exiting.\nUse -lv=LOG_INIT -w for more information.");
       simInfo->stopTime = simInfo->startTime;
@@ -571,6 +547,7 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
     infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] steps", rt_accumulated(SIM_TIMER_STEP), rt_accumulated(SIM_TIMER_STEP)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
     infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] creating output-file", rt_accumulated(SIM_TIMER_OUTPUT), rt_accumulated(SIM_TIMER_OUTPUT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
     infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] event-handling", rt_accumulated(SIM_TIMER_EVENT), rt_accumulated(SIM_TIMER_EVENT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] jacobian calculation", rt_accumulated(SIM_TIMER_JACOBIAN), rt_accumulated(SIM_TIMER_JACOBIAN)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
     infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] overhead", rt_accumulated(SIM_TIMER_OVERHEAD), rt_accumulated(SIM_TIMER_OVERHEAD)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
 
     if(S_OPTIMIZATION != solverInfo->solverMethod)
@@ -645,18 +622,17 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
  *  \param [in]  [pOptiMethod] user defined optimization method
  *  \param [in]  [pInitFile] extra argument for initialization-method "file"
  *  \param [in]  [initTime] extra argument for initialization-method "file"
- *  \param [in]  [lambda_steps] ???
  *  \param [in]  [solverID] selects the ode solver
  *  \param [in]  [outputVariablesAtEnd] ???
  *
  *  This is the main function of the solver, it performs the simulation.
  */
 int solver_main(DATA* data, threadData_t *threadData, const char* init_initMethod, const char* init_file,
-    double init_time, int lambda_steps, int solverID, const char* outputVariablesAtEnd, const char *argv_0)
+    double init_time, int solverID, const char* outputVariablesAtEnd, const char *argv_0)
 {
   TRACE_PUSH
 
-  int i, retVal = 0, initSolverInfo = 0;
+  int i, retVal = 1, initSolverInfo = 0;
   unsigned int ui;
   SOLVER_INFO solverInfo;
   SIMULATION_INFO *simInfo = data->simulationInfo;
@@ -668,12 +644,9 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
   switch(solverInfo.solverMethod)
   {
 #ifndef WITH_SUNDIALS
-  case S_RADAU1:
-  case S_RADAU3:
-  case S_RADAU5:
-  case S_LOBATTO2:
-  case S_LOBATTO4:
-  case S_LOBATTO6:
+  case S_IMPEULER:
+  case S_TRAPEZOID:
+  case S_IMPRUNGEKUTTA:
     warningStreamPrint(LOG_STDOUT, 0, "Sundial/kinsol is needed but not available. Please choose other solver.");
     TRACE_POP
     return 1;
@@ -699,6 +672,9 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
     simInfo->stepSize = MINIMAL_STEP_SIZE;
     simInfo->numSteps = round((simInfo->stopTime - simInfo->startTime)/simInfo->stepSize);
   }
+#if !defined(OMC_EMCC)
+    MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
 
   /*  initialize external input structure */
   externalInputallocate(data);
@@ -707,13 +683,9 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
 
   omc_alloc_interface.collect_a_little();
   /* initialize all parts of the model */
-  retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time, lambda_steps);
+  retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time);
   omc_alloc_interface.collect_a_little();
 
-#if !defined(OMC_EMCC)
-    MMC_TRY_INTERNAL(simulationJumpBuffer)
-#endif
-
   if(0 == retVal) {
     retVal = initializeSolverData(data, threadData, &solverInfo);
     initSolverInfo = 1;
@@ -722,9 +694,26 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
 #if !defined(OMC_MINIMAL_RUNTIME)
   dllHandle = embedded_server_load_functions(omc_flagValue[FLAG_EMBEDDED_SERVER]);
   omc_real_time_sync_init(threadData, data);
-  data->embeddedServerState = embedded_server_init(data, data->localData[0]->timeValue, solverInfo.currentStepSize, argv_0, omc_real_time_sync_update);
+  int port = 4841;
+  /* If an embedded server is specified */
+  if (dllHandle != NULL) {
+    if (omc_flag[FLAG_EMBEDDED_SERVER_PORT]) {
+      port = atoi(omc_flagValue[FLAG_EMBEDDED_SERVER_PORT]);
+      /* In case of a bad conversion, don't spawn a server on port 0...*/
+      if (port == 0) {
+        port = 4841;
+      }
+    }
+  }
+  data->embeddedServerState = embedded_server_init(data, data->localData[0]->timeValue, solverInfo.currentStepSize, argv_0, omc_real_time_sync_update, port);
+  /* If an embedded server is specified */
+  if (dllHandle != NULL) {
+    infoStreamPrint(LOG_STDOUT, 0, "The embedded server is initialized.");
+  }
+  wait_for_step(data->embeddedServerState);
 #endif
   if(0 == retVal) {
+    retVal = -1;
     /* if the model has no time changing variables skip the main loop*/
     if(data->modelData->nVariablesReal == 0    &&
        data->modelData->nVariablesInteger == 0 &&
@@ -739,7 +728,7 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
       solverInfo.currentTime = simInfo->stopTime;
       data->localData[0]->timeValue = simInfo->stopTime;
       overwriteOldSimulationData(data);
-      finishSimulation(data, threadData, &solverInfo, outputVariablesAtEnd);
+      retVal = finishSimulation(data, threadData, &solverInfo, outputVariablesAtEnd);
     } else if(S_QSS == solverInfo.solverMethod) {
       /* starts the simulation main loop - special solvers */
       sim_result.emit(&sim_result,data,threadData);
@@ -772,7 +761,7 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
       retVal = data->callback->performSimulation(data, threadData, &solverInfo);
       omc_alloc_interface.collect_a_little();
       /* terminate the simulation */
-      if (solverInfo.solverMethod == S_SYM_IMP_EULER) data->callback->symEulerUpdate(data, 0);
+      //if (solverInfo.solverMethod == S_SYM_SOLVER_SSC) data->callback->symbolicInlineSystems(data, threadData, 0, 2);
       finishSimulation(data, threadData, &solverInfo, outputVariablesAtEnd);
       omc_alloc_interface.collect_a_little();
     }
@@ -801,6 +790,9 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
     freeSolverData(data, &solverInfo);
   }
 
+  if (!retVal)
+    infoStreamPrint(LOG_STDOUT, 0, "The simulation finished successfully.");
+
   TRACE_POP
   return retVal;
 }
@@ -957,32 +949,62 @@ static int rungekutta_step_ssc(DATA* data, threadData_t *threadData, SOLVER_INFO
   return 0;
 }
 
-/***************************************    SYM_EULER_IMP     *********************************/
-static int sym_euler_im_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo){
-  int retVal,i,j;
-  /*time*/
-  solverInfo->currentTime = data->localData[1]->timeValue + solverInfo->currentStepSize;
-  data->localData[0]->timeValue = solverInfo->currentTime;
-  /*update dt*/
-  retVal = data->callback->symEulerUpdate(data, solverInfo->currentStepSize);
-  if(retVal != 0){
-    errorStreamPrint(LOG_STDOUT, 0, "Solver %s disabled on this configuration, set compiler flag +symEuler!", SOLVER_METHOD_NAME[solverInfo->solverMethod]);
-    EXIT(0);
-  }
-  /* read input vars */
-  externalInputUpdate(data);
-  data->callback->input_function(data, threadData);
-  /* eval alg equations, note ode is empty */
-  data->callback->functionODE(data, threadData);
-  /* update der(x)*/
-  for(i=0, j=data->modelData->nStates; i<data->modelData->nStates; ++i, ++j)
-    data->localData[0]->realVars[j] = (data->localData[0]->realVars[i]-data->localData[1]->realVars[i])/solverInfo->currentStepSize;
 
-  /* save stats */
-  /* steps */
-  solverInfo->solverStatsTmp[0] += 1;
-  /* function ODE evaluation is done directly after this */
-  solverInfo->solverStatsTmp[1] += 1;
+/***************************************    SYM_SOLVER     *********************************/
+static int sym_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo){
+  int retVal,i,j;
+
+  modelica_integer nStates = data->modelData->nStates;
+  SIMULATION_DATA *sData = data->localData[0];
+  SIMULATION_DATA *sDataOld = data->localData[1];
+  modelica_real* stateDer = sDataOld->realVars + data->modelData->nStates;
+
+  if (solverInfo->currentStepSize >= DASSL_STEP_EPS){
+    /* time */
+    solverInfo->currentTime = sDataOld->timeValue + solverInfo->currentStepSize;
+    sData->timeValue = solverInfo->currentTime;
+    /* update dt */
+    data->simulationInfo->inlineData->dt = solverInfo->currentStepSize;
+    /* copy old states to workspace */
+    memcpy(data->simulationInfo->inlineData->algOldVars, sDataOld->realVars, nStates * sizeof(double));
+    memcpy(sData->realVars, sDataOld->realVars, nStates * sizeof(double));
+
+    /* read input vars */
+    externalInputUpdate(data);
+    data->callback->input_function(data, threadData);
+    retVal = data->callback->symbolicInlineSystems(data, threadData);
+
+    if(retVal != 0){
+      return -1;
+    }
+
+
+    /* update der(x) */
+    for(i=0; i<nStates; ++i, ++j)
+    {
+      stateDer[i] = (sData->realVars[i]-data->simulationInfo->inlineData->algOldVars[i])/solverInfo->currentStepSize;
+    }
+
+    /* save stats */
+    /* steps */
+    solverInfo->solverStatsTmp[0] += 1;
+    /* function ODE evaluation is done directly after this */
+    solverInfo->solverStatsTmp[1] += 1;
+  }
+  else
+  /* in case desired step size is too small */
+  {
+    infoStreamPrint(LOG_SOLVER, 0, "Desired step to small try next one");
+    infoStreamPrint(LOG_SOLVER, 0, "Interpolate linear");
+
+    /* explicit euler step*/
+    for(i = 0; i < nStates; i++)
+    {
+      sData->realVars[i] = sDataOld->realVars[i] + stateDer[i] * solverInfo->currentStepSize;
+    }
+    sData->timeValue = solverInfo->currentTime + solverInfo->currentStepSize;
+    solverInfo->currentTime = sData->timeValue;
+  }
   return retVal;
 }
 
