@@ -442,6 +442,10 @@ void ShapeAnnotation::createActions()
   mpShapeAttributesAction = new QAction(Helper::attributes, mpGraphicsView);
   mpShapeAttributesAction->setStatusTip(tr("Shows the shape attributes"));
   connect(mpShapeAttributesAction, SIGNAL(triggered()), SLOT(showShapeAttributes()));
+  // edit transition action
+  mpEditTransitionAction = new QAction(Helper::editTransition, mpGraphicsView);
+  mpEditTransitionAction->setStatusTip(tr("Edits the transition"));
+  connect(mpEditTransitionAction, SIGNAL(triggered()), SLOT(editTransition()));
 }
 
 /*!
@@ -473,30 +477,42 @@ QRectF ShapeAnnotation::getBoundingRect() const
 }
 
 /*!
-  Applies the shape line pattern.
-  \param painter - pointer to QPainter
-  */
+ * \brief ShapeAnnotation::applyLinePattern
+ * Applies the shape line pattern.
+ * \param painter - pointer to QPainter
+ */
 void ShapeAnnotation::applyLinePattern(QPainter *painter)
 {
-  qreal thicknessFactor = mLineThickness / 0.5;
-  qreal thickness = thicknessFactor < 1 ? 1.0 : thicknessFactor;
-  QPen pen(mLineColor, thickness, StringHandler::getLinePatternType(mLinePattern), Qt::SquareCap, Qt::MiterJoin);
-  /* Ticket #3222
-   * Make all the shapes use cosmetic pens so that they perserve their pen widht when scaled i.e zoomed in/out.
-   * Only shapes with border patterns raised & sunken don't use cosmetic pens. We need better handling of border patterns.
+  qreal thickness = Utilities::convertMMToPixel(mLineThickness);
+  /* Ticket #4490
+   * The specification doesn't say anything about it.
+   * But just to keep this consist with Dymola set a default line thickness for border patterns raised & sunken.
+   * We need better handling of border patterns.
    */
-  if (mBorderPattern != StringHandler::BorderRaised && mBorderPattern != StringHandler::BorderSunken) {
-    pen.setCosmetic(true);
+  if (mBorderPattern == StringHandler::BorderRaised || mBorderPattern == StringHandler::BorderSunken) {
+    thickness = Utilities::convertMMToPixel(0.25);
   }
-  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
-    /* Ticket #2272, Ticket #2268.
-     * If thickness is greater than 2 then don't make the pen cosmetic since cosmetic pens don't change the width with respect to zoom.
-     */
-    if (thickness <= 2) {
-      pen.setCosmetic(true);
-    } else {
-      pen.setCosmetic(false);
-    }
+  QPen pen(mLineColor, thickness, StringHandler::getLinePatternType(mLinePattern), Qt::SquareCap, Qt::MiterJoin);
+  /* The specification doesn't say anything about it.
+   * But just to keep this consist with Dymola we use Qt::BevelJoin for Line shapes.
+   * All other shapes use Qt::MiterJoin
+   */
+  if (dynamic_cast<LineAnnotation*>(this)) {
+    pen.setJoinStyle(Qt::BevelJoin);
+  }
+  /* Ticket #3222
+   * Make all the shapes use cosmetic pens so that they perserve their pen width when scaled i.e zoomed in/out.
+   */
+  pen.setCosmetic(true);
+  /* Ticket #2272, Ticket #2268.
+   * If thickness is greater than 2 then don't make the pen cosmetic since cosmetic pens don't change the width with respect to zoom.
+   */
+  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap() && thickness > 2) {
+    pen.setCosmetic(false);
+  }
+  // if thickness is greater than 1 pixel then use antialiasing.
+  if (thickness > 1) {
+    painter->setRenderHint(QPainter::Antialiasing);
   }
   painter->setPen(pen);
 }
@@ -507,38 +523,35 @@ void ShapeAnnotation::applyLinePattern(QPainter *painter)
   */
 void ShapeAnnotation::applyFillPattern(QPainter *painter)
 {
-  switch (mFillPattern)
-  {
+  QRectF boundingRectangle;
+  if (dynamic_cast<PolygonAnnotation*>(this)) {
+    boundingRectangle = boundingRect();
+  } else {
+    boundingRectangle = getBoundingRect();
+  }
+  QLinearGradient linearGradient;
+  QRadialGradient radialGradient;
+  switch (mFillPattern) {
     case StringHandler::FillHorizontalCylinder:
-    {
-      QLinearGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().center().x(), getBoundingRect().y());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      linearGradient = QLinearGradient(boundingRectangle.center().x(), boundingRectangle.top(), boundingRectangle.center().x(), boundingRectangle.bottom());
+      linearGradient.setColorAt(0.0, mLineColor);
+      linearGradient.setColorAt(0.5, mFillColor);
+      linearGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(linearGradient);
       break;
-    }
     case StringHandler::FillVerticalCylinder:
-    {
-      QLinearGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().x(), getBoundingRect().center().y());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      linearGradient = QLinearGradient(boundingRectangle.left(), boundingRectangle.center().y(), boundingRectangle.right(), boundingRectangle.center().y());
+      linearGradient.setColorAt(0.0, mLineColor);
+      linearGradient.setColorAt(0.5, mFillColor);
+      linearGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(linearGradient);
       break;
-    }
     case StringHandler::FillSphere:
-    {
-      QRadialGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().width());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      //gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      radialGradient = QRadialGradient(boundingRectangle.center().x(), boundingRectangle.center().y(), boundingRectangle.width());
+      radialGradient.setColorAt(0.0, mFillColor);
+      radialGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(radialGradient);
       break;
-    }
     case StringHandler::FillSolid:
       painter->setBrush(QBrush(mFillColor, StringHandler::getFillPatternType(mFillPattern)));
       break;
@@ -615,8 +628,11 @@ void ShapeAnnotation::drawCornerItems()
     for (int i = 0 ; i < mPoints.size() ; i++) {
       QPointF point = mPoints.at(i);
       CornerItem *pCornerItem = new CornerItem(point.x(), point.y(), i, this);
-      /* if line is a connection then make the first and last point non moveable. */
-      if ((lineType == LineAnnotation::ConnectionType) && (i == 0 || i == mPoints.size() - 1)) {
+      /* if line is a connection or transition then make the first and last point non movable.
+       * if line is initial state then make the first point non movable.
+       */
+      if ((((lineType == LineAnnotation::ConnectionType || lineType == LineAnnotation::TransitionType) && (i == 0 || i == mPoints.size() - 1))
+           || (lineType == LineAnnotation::InitialStateType && i == 0))) {
         pCornerItem->setFlag(QGraphicsItem::ItemIsMovable, false);
       }
       mCornerItemsList.append(pCornerItem);
@@ -783,7 +799,7 @@ QVariant ShapeAnnotation::getDynamicValue(QString name)
       VariablesTreeModel *pVariablesTreeModel = pMainWindow->getVariablesWidget()->getVariablesTreeModel();
       VariablesTreeItem *pVariablesTreeItem = pVariablesTreeModel->findVariablesTreeItem(fullName, pVariablesTreeModel->getRootVariablesTreeItem());
       if (pVariablesTreeItem != NULL) {
-        dynamicValue = pVariablesTreeItem->getValue(pVariablesTreeItem->getUnit(), pMainWindow->getOMCProxy());
+        dynamicValue = pVariablesTreeItem->getValue(pVariablesTreeItem->getPreviousUnit(), pVariablesTreeItem->getUnit());
       }
     }
   }
@@ -909,6 +925,9 @@ void ShapeAnnotation::adjustCornerItemsConnectedIndexes()
 void ShapeAnnotation::removeRedundantPointsGeometriesAndCornerItems()
 {
   for (int i = 0 ; i < mPoints.size() ; i++) {
+    if (mPoints.size() <= 2) {
+      break;
+    }
     if ((i+1 < mPoints.size() && mPoints[i].y() == mPoints[i + 1].y() && i+2 < mPoints.size() && mPoints[i + 1].y() == mPoints[i + 2].y()) ||
         (i+1 < mPoints.size() && mPoints[i].x() == mPoints[i + 1].x() && i+2 < mPoints.size() && mPoints[i + 1].x() == mPoints[i + 2].x())) {
       mPoints.removeAt(i + 1);
@@ -1007,6 +1026,9 @@ void ShapeAnnotation::updateVisible()
  */
 void ShapeAnnotation::manhattanizeShape(bool addToStack)
 {
+  if (mSmooth == StringHandler::SmoothBezier) {
+    return;
+  }
   QString oldAnnotation = getOMCShapeAnnotation();
   int startIndex = -1;
   for (int i = 0 ; i < mPoints.size() ; i++) {
@@ -1097,6 +1119,10 @@ void ShapeAnnotation::referenceShapeChanged()
       updateShape(pShapeAnnotation);
       setPos(mOrigin);
       setRotation(mRotation);
+      if (dynamic_cast<TextAnnotation*>(this)) {
+        TextAnnotation *pTextAnnotation = dynamic_cast<TextAnnotation*>(this);
+        pTextAnnotation->updateTextString();
+      }
       update();
       mpParentComponent->shapeUpdated();
     }
@@ -1119,18 +1145,6 @@ void ShapeAnnotation::referenceShapeDeleted()
   }
 }
 
-/*!
-  Slot activated when Delete option is chosen from context menu of the shape.\n
-  Deletes the connection.
-  */
-void ShapeAnnotation::deleteConnection()
-{
-  LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(this);
-  if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
-    mpGraphicsView->deleteConnection(pLineAnnotation);
-  }
-}
-
 /*!
  * \brief ShapeAnnotation::deleteMe
  * Deletes the shape. Slot activated when Del key is pressed while the shape is selected.\n
@@ -1139,7 +1153,16 @@ void ShapeAnnotation::deleteConnection()
 void ShapeAnnotation::deleteMe()
 {
   // delete the shape
-  mpGraphicsView->deleteShape(this);
+  LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
+    mpGraphicsView->deleteConnection(pLineAnnotation);
+  } else if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::TransitionType) {
+    mpGraphicsView->deleteTransition(pLineAnnotation);
+  } else if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::InitialStateType) {
+    mpGraphicsView->deleteInitialState(pLineAnnotation);
+  } else {
+    mpGraphicsView->deleteShape(this);
+  }
 }
 
 /*!
@@ -1528,7 +1551,7 @@ bool ShapeAnnotation::isLineStraight(QPointF point1, QPointF point2)
  */
 void ShapeAnnotation::showShapeProperties()
 {
-  if (!mpGraphicsView || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType() == LibraryTreeItem::MetaModel) {
+  if (!mpGraphicsView || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType() == LibraryTreeItem::CompositeModel) {
     return;
   }
   MainWindow *pMainWindow = MainWindow::instance();
@@ -1546,9 +1569,24 @@ void ShapeAnnotation::showShapeAttributes()
     return;
   }
   LineAnnotation *pConnectionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
-  MetaModelConnectionAttributes *pMetaModelConnectionAttributes;
-  pMetaModelConnectionAttributes = new MetaModelConnectionAttributes(mpGraphicsView, pConnectionLineAnnotation, true, MainWindow::instance());
-  pMetaModelConnectionAttributes->exec();
+  CompositeModelConnectionAttributes *pCompositeModelConnectionAttributes;
+  pCompositeModelConnectionAttributes = new CompositeModelConnectionAttributes(mpGraphicsView, pConnectionLineAnnotation, true, MainWindow::instance());
+  pCompositeModelConnectionAttributes->exec();
+}
+
+/*!
+ * \brief ShapeAnnotation::editTransition
+ * Slot activated when edit transition option is chosen from context menu of the transition.
+ */
+void ShapeAnnotation::editTransition()
+{
+  if (!mpGraphicsView) {
+    return;
+  }
+  LineAnnotation *pTransitionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  CreateOrEditTransitionDialog *pCreateOrEditTransitionDialog = new CreateOrEditTransitionDialog(mpGraphicsView, pTransitionLineAnnotation,
+                                                                                                 true, MainWindow::instance());
+  pCreateOrEditTransitionDialog->exec();
 }
 
 /*!
@@ -1583,10 +1621,20 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
   }
 
   QMenu menu(mpGraphicsView);
-  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::MetaModel) {
+  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::CompositeModel) {
     menu.addAction(mpShapeAttributesAction);
-    menu.addSeparator();
-    menu.addAction(mpAlignInterfacesAction);
+
+    //Only show align interfaces action for bidirectional connections
+    LineAnnotation *pConnectionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+    QString startName = pConnectionLineAnnotation->getStartComponentName();
+    QString endName = pConnectionLineAnnotation->getEndComponentName();
+    CompositeModelEditor *pEditor = dynamic_cast<CompositeModelEditor*>(mpGraphicsView->getModelWidget()->getEditor());
+    if(pEditor->getInterfaceCausality(startName) == StringHandler::getTLMCausality(StringHandler::TLMBidirectional) &&
+       pEditor->getInterfaceCausality(endName) == StringHandler::getTLMCausality(StringHandler::TLMBidirectional)) {
+        menu.addSeparator();
+        menu.addAction(mpAlignInterfacesAction);
+    }
+
     menu.addSeparator();
     menu.addAction(mpGraphicsView->getDeleteAction());
   } else {
@@ -1607,12 +1655,12 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
     LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
     if (pLineAnnotation) {
       lineType = pLineAnnotation->getLineType();
-      if (lineType != LineAnnotation::ConnectionType) {
+      if (lineType != LineAnnotation::ConnectionType && lineType != LineAnnotation::TransitionType) {
         menu.addAction(mpGraphicsView->getManhattanizeAction());
       }
     }
     menu.addAction(mpGraphicsView->getDeleteAction());
-    if (lineType != LineAnnotation::ConnectionType) {
+    if (lineType != LineAnnotation::ConnectionType && lineType != LineAnnotation::TransitionType) {
       menu.addAction(mpGraphicsView->getDuplicateAction());
       menu.addSeparator();
       menu.addAction(mpGraphicsView->getBringToFrontAction());
@@ -1622,6 +1670,9 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
       menu.addSeparator();
       menu.addAction(mpGraphicsView->getRotateClockwiseAction());
       menu.addAction(mpGraphicsView->getRotateAntiClockwiseAction());
+    } else if (lineType == LineAnnotation::TransitionType) {
+      menu.addSeparator();
+      menu.addAction(mpEditTransitionAction);
     }
   }
   menu.exec(pEvent->screenPos());
@@ -1648,14 +1699,12 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
       setCursor(Qt::SizeAllCursor);
       /* Only allow manipulations on shapes if the class is not a system library class OR shape is not an inherited component. */
       if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()) {
-        if (lineType == LineAnnotation::ConnectionType) {
-          connect(mpGraphicsView, SIGNAL(mouseDelete()), SLOT(deleteConnection()), Qt::UniqueConnection);
-          connect(mpGraphicsView, SIGNAL(keyPressDelete()), SLOT(deleteConnection()), Qt::UniqueConnection);
-        } else {
-          if (pLineAnnotation) {
-            connect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()), Qt::UniqueConnection);
-          }
-          connect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        if (pLineAnnotation) {
+          connect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()), Qt::UniqueConnection);
+        }
+        connect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        connect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        if (lineType == LineAnnotation::ShapeType) {
           connect(mpGraphicsView, SIGNAL(mouseDuplicate()), this, SLOT(duplicate()), Qt::UniqueConnection);
           connect(mpGraphicsView->getBringToFrontAction(), SIGNAL(triggered()), this, SLOT(bringToFront()), Qt::UniqueConnection);
           connect(mpGraphicsView->getBringForwardAction(), SIGNAL(triggered()), this, SLOT(bringForward()), Qt::UniqueConnection);
@@ -1663,7 +1712,6 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
           connect(mpGraphicsView->getSendBackwardAction(), SIGNAL(triggered()), this, SLOT(sendBackward()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(mouseRotateClockwise()), this, SLOT(rotateClockwise()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(mouseRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()), Qt::UniqueConnection);
-          connect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressDuplicate()), this, SLOT(duplicate()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressRotateClockwise()), this, SLOT(rotateClockwise()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()), Qt::UniqueConnection);
@@ -1686,14 +1734,12 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
       unsetCursor();
       /* Only allow manipulations on shapes if the class is not a system library class OR shape is not an inherited component. */
       if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()) {
-        if (lineType == LineAnnotation::ConnectionType) {
-          disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteConnection()));
-          disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteConnection()));
-        } else {
-          if (pLineAnnotation) {
-            disconnect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()));
-          }
-          disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()));
+        if (pLineAnnotation) {
+          disconnect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()));
+        }
+        disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()));
+        disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()));
+        if (lineType == LineAnnotation::ShapeType) {
           disconnect(mpGraphicsView, SIGNAL(mouseDuplicate()), this, SLOT(duplicate()));
           disconnect(mpGraphicsView->getBringToFrontAction(), SIGNAL(triggered()), this, SLOT(bringToFront()));
           disconnect(mpGraphicsView->getBringForwardAction(), SIGNAL(triggered()), this, SLOT(bringForward()));
@@ -1701,7 +1747,6 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
           disconnect(mpGraphicsView->getSendBackwardAction(), SIGNAL(triggered()), this, SLOT(sendBackward()));
           disconnect(mpGraphicsView, SIGNAL(mouseRotateClockwise()), this, SLOT(rotateClockwise()));
           disconnect(mpGraphicsView, SIGNAL(mouseRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()));
-          disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()));
           disconnect(mpGraphicsView, SIGNAL(keyPressDuplicate()), this, SLOT(duplicate()));
           disconnect(mpGraphicsView, SIGNAL(keyPressRotateClockwise()), this, SLOT(rotateClockwise()));
           disconnect(mpGraphicsView, SIGNAL(keyPressRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()));
